# TGAM Pattern Library

## Introduction

This project forms the foundation of our living style guide. It uses [Gulp](http://gulpjs.com/) tasks to generate a [static website](http://documentation.colo.theglobeandmail.com/ux-pattern-library/), which then gets [deployed](http://bamboo.colo.theglobeandmail.com/) to a web server. It also doubles as an [NPM package](https://www.npmjs.com/). `npm install` it into your projects and you'll be able to `@import` SCSS and `require()` JavaScript from the `dist` directory.

### Workflow

The following is a high-level overview of the workflow:

1. Create or refine a pattern in the library
2. Commit your code
3. Deploy your changes to the web server via [Bamboo](http://bamboo.colo.theglobeandmail.com/)
4. Review your pattern(s) with stakeholders on the [static website](http://documentation.colo.theglobeandmail.com/ux-pattern-library/)
5. When everything has been approved, increment the pattern library version number and publish it to NPM
6. Update your projects that consume the pattern library to pull in the new version
7. Each pattern in the library includes an HTML code snippet that can be copied and pasted from the static website into your project. After pulling the latest version of the pattern library into your project, you may need to update your project's HTML to reflect changes that were made in the library.

## Installation

1. If you haven't already done so, download and install [Node.js](https://nodejs.org/) (which includes the [NPM command-line tool](https://www.npmjs.com/)).
2. Open your terminal and clone this repository.
3. Navigate to the project directory.
4. Run `npm install`.
5. Run `./node_modules/.bin/gulp`.
6. The website should automatically open in Google Chrome, with this URL: `http://localhost:3000`.

## Codebase overview

The codebase is divided into the following directories:

- `dist`: Contains a distribution-ready copy of the pattern library files. The contents of this directory will be published as an NPM package, which outside projects will then be able to install as a dependency.
- `gulp`: Contains all of the [Gulp](http://gulpjs.com/) tasks that are responsible for generating the static website and copying the pattern library files into the distribution folder.
- `public`: Contains the generated static website files. The contents of this directory will be deployed to a web server.
- `src`: Contains all of the "working files". You'll spend most of your time inside of this directory.
  - `patterns`: Contains the actual pattern library files (which will be automatically copied into the `dist` directory).
  - `specimens`: Contains examples of patterns in action, which will serve as living documentation on the static website.
  - `site`: Files in this directory define the structure, behaviour and visual style of the static website. This is where you'll go to add pages and navigation links.

### Workflow

The examples in the `specimens` directory serve as living documentation for how patterns in the library are intended to be used. Each specimen lives inside of its own sub-directory and will not appear on the static website until you've manually added it to a page (see the "Creating pages" section, below).

**Note**: Each specimen directory must have a unique name, even if it's nested inside of another directory. For example, in the following scenario, you would need to rename one of the `my-specimen` directories:

```
src
  specimens
    sub-directory-1
      my-specimen
    sub-directory-2
      my-specimen    <-- Rename this!
```

### Getting started

Start by creating a new directory inside of `specimens`, with the following (empty) files inside of it:

```
src
  specimens
    my-specimen
      data.json
      markup.hbs
      notes.html
      script.js
      styles.scss
```

During the initial prototyping phase, feel free to experiment by writing arbitrary HTML markup, SCSS and JavaScript inside of the `markup.hbs`, `styles.scss` and `script.js` files. Once your pattern is ready for production, however, your code should adhere to the following rules:

### `markup.hbs` and `data.json`

The `markup.hbs` file should contain only the HTML markup that's required to create an "instance" of a pattern. Developers should be able to copy and paste this markup into their external projects and everything should "just work" (provided they've installed the pattern library NPM package and compiled the pattern library SCSS).

If you're planning to include (a.k.a. compose) your pattern inside of another pattern, then you should replace hard-coded, context-specific values (such as link text, form label text, etc.) with [Handlebars](http://handlebarsjs.com/) variables. For example:

```
<div class="user-name-field">
  <label for="user-name">Your name</label>
  <input id="user-name" type="text" placeholder="Enter your name">
</div>
```

Could become:

```
<div class="user-name-field">
  <label for="user-name">{{ label }}</label>
  <input id="user-name" type="text" placeholder="{{ placeholder }}">
</div>
```

Notice the curly braces. This syntax is used to represent variables in Handlebars. The variables' values are defined in the specimen's `data.json` file:

```
{
  "label": "Your name",
  "placeholder": "Enter your name"
}
```

Now, assuming the above specimen files exist inside of a directory named `src/specimens/user-name-field`, a different specimen should be able to include the specimen like this:

Contents of `src/specimens/login-form/markup.hbs`:

```
<div class="login">
  <h2>{{ heading }}</h2>
  <form>
    {{> user-name-field childData }}    // Handlebars partial
    <input type="submit" value="{{ submit }}">
  </form>
</div>
```

Contents of `src/specimens/login-form/data.json`:

```
{
  "heading": "Please login",
  "childData": {
    "label": "User name",
    "placeholder": "What's your user name?"
  },
  "submit": "Submit"
}
```

See the "Handlebars partials" section below, for more information.

### `notes.html`

The markup inside of this file will appear on the static website, alongside the `iframe` that displays your specimen's HTML. This file is optional.

### `styles.scss`

This file contains additional SCSS that your specimen requires for demonstration purposes on the static website (i.e. centering it on the page via `body { text-align: center; }`). It does not contain the pattern SCSS itself. Pattern SCSS always lives inside the pattern library directory (`src/patterns/tgam`), not inside the `specimens` directory.

In `styles.scss` files, we don't want to include any SCSS that directly governs the appearance of the pattern, only ancillary code that styles its container. Remember, our goal is to create fully encapsulated, standalone "Lego block" components that can be dropped into any container, in any context.

#### Media queries

Unfortunately, media queries are only able to provide information about the viewport's dimensions and not the dimensions of individual elements on the page. This presents a challenge because our pattern library components could be nested inside of elements that are less wide than the viewport (i.e. sidebar columns, grids, etc.). Our components possess no knowledge of the dimensions of their containers, so we're forced to target our media queries to the global viewport scope. [This article describes the problem in more detail](https://ianstormtaylor.com/media-queries-are-a-hack).

Someday, when [CSS element/container queries](https://www.filamentgroup.com/lab/element-query-workarounds.html) exist (or we decide to use a [JavaScript "prolyfill"](http://elementqueries.com/)), we'll be able to write truly encapsulated components. Until then, we'll have to define our patterns' breakpoints in higher-level, parent container contexts. For example:

Contents of `src/patterns/tgam/_objects.media.scss`:

```
.o-media {
  display: flex;
  align-items: flex-start;
}
.o-media__body {
  flex: 1;
}
@mixin o-media-narrow {
  .o-media {
    display: block; // Stack elements in a single column
  }
}
```

Contents of `src/specimens/multi-column-layout/styles.scss`:

```
@media all and (max-width: 650px) {
  .l-multi-column__left {
    @include o-media-narrow;
  }
}
```

Instead of defining a media query inside of the pattern SCSS itself, we provide a mixin that external SCSS can `@include` when we need to put the pattern into the "narrow" state (since the pattern itself isn't capable of knowing when this needs to happen).

### `script.js`

Although few patterns will require JavaScript, complex user interface elements like carousels and accordion menus may require click handlers to add/remove CSS classes, dispatch events, etc. The JavaScript that defines this behaviour should live inside of the pattern library (`src/patterns/tgam/javascripts`) and should be written in a modular way, using [CommonJS modules](https://webpack.github.io/docs/commonjs.html) (recommended) or [ES2015 classes](https://developer.mozilla.org/en/docs/Web/JavaScript/Reference/Classes) (untested). For example:

Contents of `src/patterns/tgam/javascripts/carousel.js`:

```
"use strict";

let element;

function init(domElement) {
  element = domElement;
  console.log("Initialized " + element);
}

module.exports = {
  init
};
```

Contents of `src/specimens/carousel/script.js`:

```
"use strict";

let Carousel = require("carousel");

let elements = document.getElementsByClassName("c-carousel");
if (elements) {
  for (let i = 0; i < elements.length; i++) {
    let instance = Object.create(Carousel);
    instance.init(elements[i]);
  }
}
```

`script.js` files should contain boilerplate code that demonstrates how to instantiate and use a pattern, not the pattern logic itself. Developers should be able to copy and paste this code into their external projects and everything should "just work" (provided they've installed the pattern library NPM package and transpiled/bundled the pattern library JavaScript).

### Handlebars partials

For each specimen in the `src/specimens` directory, two [Handlebars](http://handlebarsjs.com/) partials will automatically be registered for you. For example, if your specimen lives inside a directory named `src/specimens/fancy-sidebar`, then the following Handlebars partials should be available for you to use:

1. `{{> fancy-sidebar }}`
2. `{{> fancy-sidebar-snippet }}`

These are essentially just file includes ([read more about partials here](http://handlebarsjs.com/partials.html)).

The first partial will contain the specimen's HTML markup, and is intended to be used primarily inside of other specimens. Remember that patterns can be composed together to create more complex patterns (i.e. a `site-header` may contain a `company-logo`, etc.), following the principles of [Atomic Design](http://bradfrost.com/blog/post/atomic-web-design/).

The second partial will contain an HTML snippet that's intended to be displayed on pages in the static website. The snippet will include markup for an `iframe` and a collapsable information panel. This partial takes several optional arguments:

- `initialWidth`: allows you to set the default width of the `iframe`.
- `fixedHeight`: allows you to apply a fixed height to the `iframe`.
- `noResize`: setting this to `true` will remove the iframe's resize handles, giving the iframe a fixed width of 320 pixels. You can use `initialWidth` to override the default and give the iframe a different width.
- `hideHtml`: setting this to `true` will prevent the specimen's HTML markup from appearing inside of the collapsable information panel.
- `hideJs`: setting this to `true` will prevent the specimen's JavaScript code from appearing inside of the collapsable information panel.

For example, you could set the width of the `fancy-sidebar` specimen's `iframe` and hide it's HTML like so:

`{{> fancy-sidebar-snippet initialWidth="400px" hideHtml=true }}`

Please note that you must include your snippet on a page before it will become visible on the static website (see the "Creating pages" section, below).

## Previewing your work

In order to preview your work, you'll need to compile the static website by running the Gulp task (`./node_modules/.bin/gulp`). While it's running, Gulp should monitor your files for changes and recompile your SASS as required. It will also start up a local web server, which will make it possible to view the static website by navigating to http://localhost:3000 in your web browser.

If you encounter a situation where Gulp isn't recompiling properly, then just stop it via <kbd>control + c</kbd> (on your keyboard) and restart it via `./node_modules/.bin/gulp` (in your terminal). Please note that Gulp is only able to monitor the files that existed when its `watch` tasks started up, so you'll need to restart it when you create new files and directories.

## Creating pages

To add a new page to the static website, create a new Handlebars template (an HTML file with an `.hbs` extension) inside of the `src/site/pages/` directory. When the Gulp tasks run, the contents of this template file will be inserted into the `<body>` of the generated page. You can use Handlebars partials to pull in specimen snippets, like this: `{{> my-specimen-snippet }}`, where `my-specimen` is the name of the specimen directory (see the "Handlebars partials" section, above). Pages also contain a `metaData` HTML code comment, which you can use to set the page's title and meta title. So a complete page could look something like this:

```
<!--metaData
metaTitle: Home | My fancy website
pageTitle: Home page
metaData-->

<h2>Check out my fancy component</h2>

{{> fancy-component-snippet }}

<p>Isn't it great?</p>
```

Pages that you create inside of `src/site/pages/` will automatically end up inside the root directory of the static website. For example:

- `/src/site/pages/my-page.hbs` will become `/my-page.html`
- `/src/site/pages/cat-videos/index.hbs` will become `/cat-videos/index.html`

## Adding links to the menu

To add a new link to the static website's navigation menu, edit the `src/site/navigation.yml` file. This file defines the structure of the menu, in [YAML](http://www.yaml.org/start.html) format. It should look something like this:

```
primaryNav:
  -
    name: "Home"
    path: "/"
  -
    name: "Another page"
    path: "/another-page.html"
  -
    name: "External link"
    path: "http://www.google.com"
  -
    name: "Some category"
    path: "/some-category"
    children:
      -
        name: "Nested page"
        path: "/some-category/nested-page"
```

The above menu structure will generate HTML similar to this:

```
<ul>
  <li><a href="/">Home</a></li>
  <li><a href="/another-page.html">Another page</a></li>
  <li><a href="http://www.google.com">External link</a></li>
  <li>
    <a href="/some-category">Some category</a>
    <ul>
      <li><a href="/some-category/nested-page">Nested page</a></li>
    </ul>
  </li>
</ul>
```

**Note**: Only one level of link nesting is currently supported.

## Working with the pattern library files

The pattern library consists of the SCSS, JavaScript, fonts and images that live inside of the `src/patterns/tgam` directory:

```
src/patterns/tgam
src/patterns/tgam/fonts
src/patterns/tgam/images
src/patterns/tgam/javascripts
```

These are the files that get copied into the `dist` directory when the Gulp tasks run, and get published as an NPM package when you deploy the static website files (see the "Publishing new versions of the pattern library NPM package" section, below).

----------
### Aside

We decided not to include HTML files in the pattern library because importing HTML snippets directly into our projects would complicate our workflow. This may change in the future, of course. Here's a brief overview of the pros and cons of each approach:

#### Pattern HTML imported directly into projects

Rather than importing discrete chunks of HTML, we would create a "Component API" (a set of functions that return HTML markup). Various arguments would be passed into the Component API functions (i.e. class names, text strings, etc.), which would result in different markup being returned. Lonely Planet followed this strategy with their ["Rizzo" engine](http://rizzo.lonelyplanet.com/styleguide/ui-components/cards#styleguide-block__item--1).

**Pros**

- **Maintenance**: Easier to maintain because the HTML lives in a single place. Updates are automatically applied to all instances of the code, in every project that uses the pattern library.
- **Consistency**: Developers are not able to directly modify the code, which means it's not possible for them to implement patterns incorrectly (i.e. typos, missing classes in the markup). They also aren't able to create a million subtle variations of a single pattern, which would be difficult to maintain over time.

**Cons**

- **Frustration**: Because developers aren't able to modify the code directly, it's difficult for them to add things like `aria` and `data` attributes to individual instances of patterns. The only way for them to do this is to modify the Component API function that returns the markup (i.e. add a new argument to the function signature and write some additional logic inside the function body). Another option would be to circumvent the Component API entirely by "forking" the pattern (copying and pasting the HTML that the function returns and working with that HTML directly).

#### Pattern HTML written by hand

**Pros and cons**

- Developers are free to make subtle adjustments to pattern markup, but must remain disciplined when doing so. Care must be taken not to alter patterns too much, otherwise we could end up with a disparate array of mutated "Frankenpatterns" that are a nightmare to maintain. If a developer encounters a scenario where they need to dramatically alter an existing pattern, then they are free to do so in order to satisfy their short-term requirements. However, they should bring this to the attention of the UX Design team and collaborate with them to design a proper, documented, tested pattern to replace their non-standard solution.

----------

## CSS methodology

The CSS in this project is organized according to the [ITCSS](http://itcss.io/) (Inverted Triangle CSS) methodology. If you aren't already familiar with ITCSS, then please [watch this video](https://www.youtube.com/watch?v=1OKZOV-iLj4).

```
\ ------------------------------------ /
 \              Settings              /
  \ -------------------------------- /
   \             Tools              /
    \ ---------------------------- /
     \          Generic           /
      \ ------------------------ /
       \          Base          /
        \ -------------------- /
         \      Objects       /
          \ ---------------- /
           \   Components   /
            \ ------------ /
             \   Trumps   /
              \ -------- /
```

### Import order

ITCSS concerns itself primarily with CSS specificity. This means that you should strive to organize your CSS into the following categories. Each category is numbered and corresponds to a layer in the inverted triangle diagram:

1. **Requirements** -- Vendor libraries
2. **Settings** ------ Global variables, site-wide settings, config switches, etc.
3. **Tools** --------- Site-wide mixins and functions.
4. **Generic** ------- "Ground-zero" styles. Low-specificity, far-reaching rulesets (e.g. normalize, resets, box-sizing).
5. **Base** ---------- Unclassed HTML elements (type selectors: `a {}`, `blockquote {}`, `address {}`, etc.)
6. **Objects** ------- Objects, abstractions, and "cosmetic-free" design patterns (e.g. `.media {}`).
7. **Components** ---- Designed components. Discrete, complete chunks of UI (e.g. `.carousel {}`).
8. **Trumps** -------- High-specificity, very explicit selectors. Overrides and helper classes (e.g. `.hidden {}`).

### Differences between objects and components

Generally speaking, Objects provide the structure and Components provide the "skin". In Object-Oriented Programming terminology, Objects are like abstract classes and Components are like concrete classes. For example, an *Author Profile Block* is a type of *Media Object*, so the *Media Object* would be an Object (abstract class) and the *Author Profile Block* would be a Component (concrete class).

### Resources

You may find it helpful to refer to the following resources for ideas on how to organize your CSS:

- [InuitCSS documentation](https://github.com/inuitcss/getting-started#import-order)
- [InuitCSS repository](https://github.com/inuitcss)
- [Files from Harry Roberts' .Net Magazine screencast](https://github.com/itcss/itcss-netmag)
- [Files from one of Harry Roberts' projects](https://github.com/csswizardry/frcss)

## Releasing new versions of the pattern library

In order to release a new version of the pattern library, you'll need to log in to Bamboo and run the build plan:

1. [Log in to Bamboo](http://bamboo.colo.theglobeandmail.com/userlogin!default.action)
2. Select "Build -> All build plans" from the menu.
3. Find "ux-pattern-library" in the list.
4. Click the "Run" icon in the left-hand side of the table row.

The build plan will do the following:

- Redeploy the static website files to the web server.
- Increment the NPM package's version number in `package.json`.
- Republish the NPM package to the NPM registry.
- Generate a Git tag with the same version number as the one that's defined inside `package.json`.

### Development workflow

It's best practice to create a new Git branch before you start making changes to the codebase:

1. Check out the `master` branch: `git checkout master`.
2. Pull down the latest changes: `git pull`.
3. Create a new working branch: `git checkout -b my-new-branch`.
4. Push your local branch up to the remote repository: `git push -u origin my-new-branch`.
5. Do some work and commit your changes: `git commit -m "My commit message"`.
6. Repeat step 5 as many times as necessary.
7. When you're finished working, go to Github.com and [create a new pull request](https://github.com/globeandmail/ux-pattern-library/pulls).
8. Wait for your peers to review your pull request and merge it into `master`.
9. [Log in to Bamboo](http://bamboo.colo.theglobeandmail.com/userlogin!default.action) and run the build plan (see above).

## Importing the pattern library NPM package into outside projects

In order to make use of the pattern library, you'll need to install it as a dependency in your project(s).

1. If you haven't already done so, install a task runner such as [Gulp](http://gulpjs.com/) (recommended) or [Grunt](http://gruntjs.com/) (untested).
2. If you haven't already done so, initialize your project via `npm init`.
3. Install the pattern library module via `npm install --save-dev tgam-patterns`.
4. Install the `node-sass-asset-functions` module via `npm install --save-dev node-sass-asset-functions`. This module will allow us to process the `font-url()` and `image-url()` SCSS helpers that the pattern library utilizes.
5. Create a file called `gulpfile.js` (or edit your existing `gulpfile.js`) and add the following:

```
var gulp = require("gulp");
var sass = require("gulp-sass");
var assetFunctions = require("node-sass-asset-functions");
var tgamIncludePath = require("tgam-patterns").sassIncludePath;

// Copy font and image files from node_modules into a deployable directory

gulp.task("tgam-patterns:fonts", function() {
  return gulp.src("./node_modules/tgam-patterns/dist/tgam/fonts/**/*.*")
    .pipe(gulp.dest("./path/to/your/fonts"));
});

gulp.task("tgam-patterns:images", function() {
  return gulp.src("./node_modules/tgam-patterns/dist/tgam/images/**/*.*")
    .pipe(gulp.dest("./path/to/your/images"));
});

gulp.task("tgam-patterns", ["tgam-patterns:fonts", "tgam-patterns:images"], function(done) {
  done();
});

// Compile SASS files

gulp.task("styles:compile", function() {
  return gulp.src("./path/to/your/main/stylesheet.scss")
    .pipe(sass({
      includePaths: tgamIncludePath, // SASS include paths
      functions: assetFunctions({
        // Specify base URLs for SASS helpers (font-url(), image-url(), etc.)
        images_path: "/path/to/your/images",
        fonts_path: "/path/to/your/fonts",
        http_images_path: "/path/to/your/images",
        http_fonts_path: "/path/to/your/fonts"
      })
    })
    .pipe(gulp.dest("./path/to/your/output/directory"));
});
```

You should now be able to import patterns into your SCSS files via `@import "tgam/all";` and compile them via `gulp styles:compile`. You'll also need to run `gulp tgam-patterns` after installing the pattern library module.

### Updating the NPM package

1. Run `npm outdated` in your command line to see if a newer version of the pattern library is available.
2. Edit `package.json` and manually increment the `tgam-patterns` version number to the latest.
3. Run `npm update tgam-patterns`.
4. Double check that the `node_modules/tgam-patterns/dist` directory exists. If not, then someone forgot to run the Gulp tasks before publishing this version of the pattern library.

**Note**: Unfortunately, it's necessary to manually edit `package.json` because running `npm-update` alone will not update the version number in your `package.json` file, nor will running `npm install --save tgam-patterns`.

### TODO

- Make specimen title bar (with buttons) stick to top of viewport so that it scrolls with the user (like sticky table headers)
- Make it possible to hydrate pattern library specimens with different data, at the click of a button (there would be multiple data.json files or perhaps multiple objects inside the single data.json file. Each would include a different set of data, so designers and developers could preview what patterns will look like with long text, short text, inline links, etc. A toggle or dropdown would be added to the UI, beside the font toggle button.)
- Research what it would take to embed CodePen or JSBin into pattern library app
- Add iframe reload button to specimen button panel
- Add optional `javascripts` and `stylesheets` arrays to specimen `metaData` blocks, then pull these assets into the specimen's HTML page (external CDN-hosted assets with absolute URLs, local assets in the repo or modules in the `node_modules` directory?)
- Create Gulp task that generates an empty Specimen directory with all of the required (empty) files inside
- Add a third resize handle to the bottom of the iframe, to make it possible to adjust the height
