"use strict";

const ee = require("./eventemitter");

const dialogClass = "c-modal-dialog";
const eventNamespace = "tgam-modal-dialog";

module.exports = {

  init: function(openButtons, dialog) {
    this.dialog = dialog;
    this.closeButton = dialog.getElementsByClassName("c-modal-dialog__close-button").item(0);
    this.openButtons = [].slice.call(openButtons);
    this.openButtonThatSpawnedDialog;

    // Ensure that the "this" keyword always references the correct
    // object when the following methods are used as event handlers
    this.boundOnOpenButtonClick = this.onOpenButtonClick.bind(this);
    this.boundOnKeyPress = this.onKeyPress.bind(this);
    this.boundOnFocusChange = this.onFocusChange.bind(this);

    if (this.dialog && this.openButtons.length && this.closeButton) {
      this.openButtons.map(openButton => {
        openButton.addEventListener("click", this.boundOnOpenButtonClick);
      });
      this.closeButton.addEventListener("click", () => {
        this.close();
      });
      this.dialog.addEventListener("animationend", () => {
        this.onDialogAnimationEnd();
      });
    }
  },

  isOpen: function() {
    return this.openButtons[0].getAttribute("aria-expanded") === "true";
  },

  open: function() {
    this.openButtons.map(openButton => {
      openButton.setAttribute("aria-expanded", true);
    });
    this.dialog.classList.remove(`${dialogClass}--hiding`);
    this.dialog.classList.add(`${dialogClass}--revealing`);
    this.dialog.removeAttribute("hidden");
    this.addKeyPressListener();
    this.activateKeyboardTrap();
    ee.emitEvent(`${eventNamespace}:opening`);
  },

  close: function() {
    this.openButtons.map(openButton => {
      openButton.setAttribute("aria-expanded", false);
    });
    this.dialog.classList.remove(`${dialogClass}--revealing`);
    this.dialog.classList.add(`${dialogClass}--hiding`);
    this.removeKeyPressListener();
    this.deactivateKeyboardTrap();

    if (this.openButtonThatSpawnedDialog) {
      this.openButtonThatSpawnedDialog.focus();
      this.openButtonThatSpawnedDialog = null;
    } else {
      this.openButtons[0].focus();
    }
    ee.emitEvent(`${eventNamespace}:closing`);
  },

  onDialogAnimationEnd: function(e) {
    // Using classList.remove() with multiple arguments fails in IE11,
    // so we're removing classes separately
    this.dialog.classList.remove(`${dialogClass}--hiding`);
    this.dialog.classList.remove(`${dialogClass}--revealing`);
    if (this.openButtons[0].getAttribute("aria-expanded") === "false") {
      this.dialog.setAttribute("hidden", "hidden");
      ee.emitEvent(`${eventNamespace}:closed`);
    } else {
      this.closeButton.focus();
      ee.emitEvent(`${eventNamespace}:open`);
    }
  },

  onOpenButtonClick: function(event) {
    if (event.target.nodeName.toLowerCase() === "a") {
      event.preventDefault();
    }
    this.openButtonThatSpawnedDialog = event.target;
    this.open();
  },

  addKeyPressListener: function() {
    document.addEventListener("keydown", this.boundOnKeyPress, false);
  },

  removeKeyPressListener: function() {
    document.removeEventListener("keydown", this.boundOnKeyPress);
  },

  onKeyPress: function(event) {
    if (this.isOpen() && event.keyCode === 27) { // Esc
      this.close();
    }
  },

  activateKeyboardTrap: function() {
    document.addEventListener("focus", this.boundOnFocusChange, true);
  },

  deactivateKeyboardTrap: function() {
    document.removeEventListener("focus", this.boundOnFocusChange);
  },

  onFocusChange: function(event) {
    if (this.isOpen() && !this.isDescendant(this.dialog, document.activeElement)) {
      // Set focus to the first focusable element inside the dialog
      this.closeButton.focus();
    }
  },

  // Helper method
  isDescendant: function(parent, child) {
    let node = child.parentNode;
    while (node != null) {
      if (node == parent) {
        return true;
      }
      node = node.parentNode;
    }
    return false;
  }

};
