"use strict";

/**
 * Usage (OLOO pattern)
 *
 * 1. Import this module:
 *      var RelatedArticles = require("tgam-patterns").relatedArticles;
 *
 * 2. Create a new object that links to the object that this module exports:
 *      var newObject = Object.create(RelatedArticles);
 *
 * 3. The new object will be linked to the exported object via its __proto__
 *    property (a.k.a. its prototype):
 *      console.log(newObject);
 *      // Object {
 *      //   button: Element
 *      //   drawer: Element
 *      //   __proto__: Object {
 *      //     init: Function
 *      //     open: Function
 *      //   }
 *      // }
 *
 * 4. Learn more about JavaScript's prototype chain and the OLOO pattern here:
 *    https://github.com/getify/You-Dont-Know-JS/blob/master/this%20%26%20object%20prototypes/ch5.md
 */

const ee = require("./eventemitter");

const buttonClass = "js-c-related-articles__more";
const drawerClass = "js-c-related-articles__drawer";
const eventNamespace = "tgam-related-articles";

module.exports = {

  init: function(rootElement) {
    this.button = rootElement.getElementsByClassName(buttonClass).item(0);
    this.drawer = rootElement.getElementsByClassName(drawerClass).item(0);

    if (this.button && this.drawer) {
      this.button.addEventListener("click", () => {
        this.open();
      });
      this.drawer.addEventListener("animationend", () => {
        this.onDrawerAnimationEnd();
      });
    }
  },

  open: function() {
    if (this.button.getAttribute("aria-expanded") === "false" && !this.drawer.classList.contains(`${drawerClass}--open`)) {
      this.button.setAttribute("aria-expanded", true);
      this.button.classList.add("u-hidden");
      this.drawer.classList.add(`${drawerClass}--open`);
      // The "hidden" attribute hides elements from all user agents, including screen readers
      this.drawer.removeAttribute("hidden");
      // Must assign focus to a new element after hiding the button
      this.drawer.focus();
      ee.emitEvent(`${eventNamespace}:opening`, [this.drawer]);
    }
  },

  onDrawerAnimationEnd: function(e) {
    ee.emitEvent(`${eventNamespace}:open`, [this.drawer]);
  }

};
